# -*- coding: utf-8 -*-
"""
Created on Sat Nov 14 11:22:56 2020

@author: kourf
"""

import random
import doctest
import timeit
import matplotlib.pyplot as plt


def distance(p1, p2):
    """
    Renvoie la distance entre deux points

    
    Exemple
    -------
    >>> distance((1, 2), (3, 5))
    3.605551275463989
    """



def plus_proches(points: list) -> (float, tuple, tuple): 
    """
    Recherche les deux points les plus proches d'une liste de points

    Parameters
    ----------
    points : list
        liste des points définis par leurs coordonnées 
        [(x1, y1), …, (xn, yn)] 

    Returns
    -------
    (float, tuple, tuple)
        la distance minmale et les deux points les plus proches

    
    Example
    -------
    >>> plus_proches([(3, 4), (2, -1), (0.5, 2), (3, -4), (-1, -0.5), (-2, 5)])
    (2.9154759474226504, (0.5, 2), (-1, -0.5))
    
    """



def plus_proches_dpr(points: list) -> (float, tuple, tuple): 
    """
    Même préconditions et postconditions que plus_proches
    Utilise la méthode diviser pour régner
    """
    def plus_proches_rec(Px: list, Py: list) -> (float, tuple, tuple):
        # étape 2
        # à compléter
        # étape 3
        # à compléter
        # étape 4
        # à compléter
        return # à compléter
       
    Px = sorted(points)
    Py = sorted(points, key=lambda p: p[1])
    return plus_proches_rec(Px, Py)
    
    

def test_alea(xmin, xmax, ymin, ymax, nb):
    """
    Génère aléatoirement une liste de nb points
    d'abscisses entre xmin et xmax et d'ordonnées entre ymin et ymax
    """
    res = []
    for i in range(nb):
        x = xmin + (xmax - xmin) * random.random()
        y = ymin + (ymax - ymin) * random.random()
        res.append((x, y))
    return res        


def test_plus_proches_dpr():
    # Validité
    tests = ([[(-74, 26), (-61, 47), (-62, 37), (-39, -82)]]
             + [test_alea(-100, 100, -100, 100, 10) for i in range(100)]
             + [test_alea(-100, 100, -100, 100, 201) for i in range(10)])
    for t in tests:
        res1 = plus_proches(t)
        res2 = plus_proches_dpr(t)
        if res1[0] != res2[0] or {res1[1], res1[2]} != {res2[1], res2[2]}:
            print('test failed on', t)
            print('got', res1, 'and', res2)
            return
    print('Validité ok')
    # Vitesse
    npoints = 1000
    nrepeat = 10
    print('Temps de calcul lors de {} recherches sur des nuages de {} points'.format(nrepeat, npoints))
    t1 = timeit.timeit('plus_proches(test_alea(-100, 100, -100, 100, {}))'.format(npoints), globals = globals(), number=nrepeat)
    t2 = timeit.timeit('plus_proches_dpr(test_alea(-100, 100, -100, 100, {}))'.format(npoints), globals = globals(), number=nrepeat)
    print('Recherche naïve : {} - Algorithme diviser pour régner : {}'.format(t1, t2))


def graphique_ppdpr(n):
    points = test_alea(-10, 10, -10, 10, n)
    absc = [p[0] for p in points]
    ordo = [p[1] for p in points]
    plt.scatter(absc, ordo, color='black')
    d, p1, p2 = plus_proches_dpr(points)
    #plt.plot([p1[0], p2[0]], [p1[1], p2[1]], color='red')
    plt.scatter([p1[0], p2[0]], [p1[1], p2[1]], color='red')
    plt.show()
    



if __name__ == '__main__':
    import doctest
    doctest.testmod()